<?
/**
 * Samurai_Renderer_Smarty
 * 
 * SamuraiFWとSmartyとのブリッジクラス。
 * 
 * @package    Samurai
 * @subpackage Samurai.Renderer
 * @copyright  Befool,Inc.
 * @author     Satoshi Kiuchi <satoshi.kiuchi@befool.co.jp>
 * @license    http://opensource.org/licenses/bsd-license.php  The modified BSD License
 */
Samurai_Loader::loadByClass('Samurai_Renderer');
Samurai_Loader::loadByClass('Samurai_Renderer_Engine_Smarty');
class Samurai_Renderer_Smarty extends Samurai_Renderer
{
    public
        /** @var        object  Smartyインスタンス */
        $Engine;
    
    
    /**
     * setEngine.
     * @override
     */
    protected function _setEngine()
    {
        $this->Engine = new Samurai_Renderer_Engine_Smarty();
        Samurai::getContainer()->injectDependency($this->Engine);
    }
    
    
    /**
     * renderer.
     * @override
     */
    public function render($template)
    {
        //テンプレート解釈
        $result = $this->Engine->fetch($template);
        return $result;
    }
    
    
    /**
     * assign.
     * @implements
     */
    public function assign($key, $value)
    {
        $this->Engine->assign($key, $value);
    }
    
    
    /**
     * getAssignedVars.
     * @override
     */
    public function getAssignedVars()
    {
        return $this->Engine->get_template_vars();
    }
    
    
    /**
     * Helperの登録。
     * @access     public
     * @param      string  $alias    登録名
     * @param      array   $define   宣言内容
     */
    public function addHelper($alias, $define)
    {
        $Container = Samurai::getContainer();
        $Container->registerComponent("Renderer_Smarty_{$alias}", new Samurai_Container_Def($define));
        $Helper = $Container->getComponent("Renderer_Smarty_{$alias}");
        $this->Engine->register_object($alias, $Helper);
        return $Helper;
    }
    
    
    
    
    
    /**
     * Setters.
     * @implements
     */
    public function setAction($Action)
    {
        $Action->beforeRenderer();
        $this->Engine->assign('Action', $Action);
        foreach(get_object_vars($Action) as $_key => $_val){
            if(!preg_match('/^_/', $_key)){
                $this->Engine->assign($_key, $_val);
            }
        }
    }
    public function setErrorList($ErrorList)
    {
        $this->Engine->assign('ErrorList', $ErrorList);
    }
    public function setRequest($Request)
    {
        $this->Engine->register_object('Request', $Request);
        $this->Engine->assign('Request', $Request);
        $this->Engine->assign('request', $Request->getParameters());
    }
    public function setCookie($Cookie)
    {
        $this->Engine->register_object('Cookie', $Cookie);
        $this->Engine->assign('Cookie', $Cookie);
        $this->Engine->assign('cookie', $Cookie->getParameters());
    }
    public function setSession($Session)
    {
        $this->Engine->register_object('Session', $Session);
        $this->Engine->assign('Session', $Session);
        $this->Engine->assign('session', $Session->getParameters());
    }
    public function setServer($server)
    {
        $this->Engine->assign('server', $server);
    }
    public function setScriptName($script_name)
    {
        $this->Engine->assign('script_name', $script_name);
    }
    public function setToken($Token)
    {
        $this->Engine->register_object('Token', $Token);
        $this->Engine->assign('Token', $Token);
        $this->Engine->assign('token', array('name'=>$Token->getName(), 'value'=>$Token->getValue()));
    }
}
